@@ -324,6 +324,25 @@ volume_is_blank (GnomeVFSVolume *volume)
 	return is_blank;
 }
 
+static GnomeVFSVolume *
+drive_get_first_volume (GnomeVFSDrive *drive)
+{
+	GnomeVFSVolume *volume;
+	GList          *volumes;
+
+	volumes = gnome_vfs_drive_get_mounted_volumes (drive);
+
+	volume = g_list_nth_data (volumes, 0);
+
+	if (volume != NULL) {
+		gnome_vfs_volume_ref (volume);
+	}
+
+	gnome_vfs_drive_volume_list_free (volumes);
+
+	return volume;
+}
+
 static GList *
 nautilus_burn_get_file_items (NautilusMenuProvider *provider,
 			      GtkWidget            *window,
@@ -366,7 +385,8 @@ nautilus_burn_get_file_items (NautilusMenuProvider *provider,
 
 	is_iso = (strcmp (mime_type, "application/x-iso-image") == 0)
 		|| (strcmp (mime_type, "application/x-cd-image") == 0);
-	is_cue          = strcmp (mime_type, "application/x-cue") == 0;
+	is_cue = strcmp (mime_type, "application/x-cue") == 0
+		|| (strcmp (mime_type, "application/x-cdrdao-toc") == 0);
 
 	if (is_iso && is_local) {
 		item = nautilus_menu_item_new ("NautilusBurn::write_iso",
@@ -390,12 +410,33 @@ nautilus_burn_get_file_items (NautilusMenuProvider *provider,
 		items = g_list_append (items, item);
 	}
 
+	/*
+	 * We handle two cases here.  The selection is:
+	 *  A) a volume
+	 *  B) a drive
+	 *
+	 * This is because there is very little distinction between
+	 * the two for CD/DVD media
+	 */
+
 	drive = nautilus_file_info_get_drive (file);
 	volume = nautilus_file_info_get_volume (file);
-	if (drive == NULL
-	    && volume != NULL) {
+
+	/* nautilus-file-info doesn't increment refcount */
+	if (drive != NULL) {
+		gnome_vfs_drive_ref (drive);
+	}
+	/* nautilus-file-info doesn't increment refcount */
+	if (volume != NULL) {
+		gnome_vfs_volume_ref (volume);
+	}
+
+	if (drive == NULL && volume != NULL) {
+		/* case A */
 		drive = gnome_vfs_volume_get_drive (volume);
-		gnome_vfs_drive_unref (drive);
+	} else if (volume == NULL && drive != NULL) {
+		/* case B */
+		volume = drive_get_first_volume (drive);
 	}
 
 	if (drive != NULL
@@ -418,6 +459,13 @@ nautilus_burn_get_file_items (NautilusMenuProvider *provider,
 		items = g_list_append (items, item);
 	}
 
+	if (drive != NULL) {
+		gnome_vfs_drive_unref (drive);
+	}
+	if (volume != NULL) {
+		gnome_vfs_volume_unref (volume);
+	}
+
 	g_free (mime_type);
 
 	return items;
